import re
from os import path
from concurrent import futures
from datetime import datetime, timedelta

import bs4
import requests
import urllib3
from tqdm import tqdm

from .util import TEACHERS, extract_teacher

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def param(data: dict):
    return "&".join([key + "=" + value for key, value in data.items()])


class Fetcher:

    URLS = {
        "sso-login": "https://edumate.rosebank.nsw.edu.au/rosebank/web/app.php/sso-login/",
        "login": "https://rosebank-login.cloudworkengine.net/module.php/core/loginuserpass.php?",
        "accounts": "https://edumate.rosebank.nsw.edu.au/rosebank/web/app.php/saml/acs",
        "day": "https://edumate.rosebank.nsw.edu.au/rosebank/web/app.php/admin/get-day-calendar/{}/next",
    }
    MAX_WORKERS = 20
    EDUMTE_RE = r"^\S+ (.*) (\d+|\w+|\d+\w+) \((\w\d{3}|.*).*\).*"
    EDUMATE_FMT = "%Y-%m-%d %H:%M:%S.000000"

    colour_to_code = {
        "tomato": 11,
        "flamingo": 4,
        "tangerine": 6,
        "banana": 5,
        "sage": 2,
        "basil": 10,
        "peacock": 7,
        "blueberry": 9,
        "lavender": 1,
        "grape": 3,
        "graphite": 8,
    }

    colours = {}

    def __init__(self, username: str, password: str, conf_folder):
        self.colour_conf = path.join(conf_folder, "colour_config.txt")

        if path.isfile(self.colour_conf):
            self.colours = {}
            with open(self.colour_conf) as file:
                for line in file.readlines():
                    split = line.split(":")
                    if len(split) <= 1:
                        continue
                    key, value = split
                    self.colours[key.strip()] = value.strip().lower()

                if "default" not in self.colours.keys():
                    self.colours["default"] = "graphite"
        else:
            with open(self.colour_conf, "w+") as file:
                for line in self.colours.items():
                    file.write(f"{line[0]}: {line[1]}\n")

        self.session = requests.Session()
        self.session.verify = False
        self.credentials = {"username": username, "password": password}
        self.login()
        if self.successful_login():
            print("Successful login")
        else:
            print("Login failed")
            quit()

    def login(self):
        sso_login = self.session.get(self.URLS["sso-login"])
        soup = bs4.BeautifulSoup(sso_login.history[1].content, "html.parser")
        auth = {"AuthState": soup.a.attrs["href"][soup.a.attrs["href"].find("=") + 1 :]}
        login = self.session.post(
            self.URLS["login"] + param({**auth, **self.credentials})
        )
        soup = bs4.BeautifulSoup(login.content, "html.parser")
        saml_response = {"SAMLResponse": soup.find_all("input")[1].attrs["value"]}
        self.session.post(self.URLS["accounts"], data=saml_response)

    def successful_login(self):
        test = self.session.get(self.URLS["day"].format("2019-02-01"))
        return test.status_code == 200 and test.json()

    def get_dates(self, dates: iter):
        data = []
        with futures.ThreadPoolExecutor(max_workers=self.MAX_WORKERS) as executor:
            to_do = []
            for date in dates:
                date = date - timedelta(days=1)
                date_str = date.strftime("%Y-%m-%d")
                future = executor.submit(
                    self.session.get, self.URLS["day"].format(date_str)
                )
                to_do.append(future)

            for future in tqdm(futures.as_completed(to_do), total=len(dates)):
                data.append(future.result().json())

        return data

    def get_colour(self, name):
        return self.colour_to_code.get(
            self.colours.get(name, self.colours["default"]),
            self.colour_to_code[self.colours["default"]],
        )

    def get_simple_dates(self, dates):
        formatted_data = []
        skip = []  # due to double periods
        for day in self.get_dates(dates):
            day_data = []
            for index, event in enumerate(day["events"]):
                if event in skip:
                    continue
                if event["eventType"] == "class":
                    match = re.match(self.EDUMTE_RE, event["activityName"])
                    subject = match.group(1) if match else event["activityName"]
                    room = match.group(3) if match else None
                    start = self.time_to_datetime(event["startDateTime"]["date"])
                    end = self.time_to_datetime(event["endDateTime"]["date"])
                    teacher = extract_teacher(event["links"])

                    if event["period"].isdigit() and int(event["period"]) % 2:
                        next_event = day["events"][index + 1]
                        match = re.match(self.EDUMTE_RE, next_event["activityName"])
                        next_subject = match.group(1) if match else event["activityName"]
                        if subject == next_subject:
                            skip.append(next_event)
                            next_room = match.group(3) if match else None
                            next_teacher = extract_teacher(event["links"])
                            end = self.time_to_datetime(next_event["endDateTime"]["date"])
                            if next_room != room:
                                room += f", {next_room}"
                            if next_teacher != teacher:
                                teacher += f", {next_teacher}"

                    # Edumate says extension classes start at 8:13
                    if event["period"] == "BS2":
                        start = start.replace(hour=7, minute=30)

                    event_data = {
                        "name": subject,
                        "colour": self.get_colour(subject),
                        "room": room,
                        "start_time": start,
                        "end_time": end,
                        "description": teacher,
                        "timezone": event["startDateTime"]["timezone"].title(),
                    }

                elif event["eventType"] == "event":
                    match = re.match(self.EDUMTE_RE, event["activityName"])
                    event_data = {
                        "name": match.group(1) + " " + match.group(2),
                        "colour": self.get_colour(event["activityName"].split(" ")[1]),
                        "start_time": self.time_to_datetime(
                            event["startDateTime"]["date"]
                        ),
                        "description": None,
                        "end_time": self.time_to_datetime(event["endDateTime"]["date"]),
                        "timezone": event["startDateTime"]["timezone"].title(),
                        "room": None,
                        "colour": self.get_colour("event"),
                    }
                else:
                    event_data = {}

                day_data.append(event_data)

            formatted_data.append(day_data)

        return formatted_data

    def get_week(self, date):
        monday = date - timedelta(days=date.weekday())
        week = [monday + timedelta(days=day_int) for day_int in range(-1, 4)]
        week_data = self.get_simple_dates(week)
        return week_data

    def time_to_datetime(self, time):
        # "2019-02-01 08:42:00.000000"
        return datetime.strptime(time, self.EDUMATE_FMT)

    def __repr__(self):
        return f"<Fetcher username={self.credentials['username']} password=...>"



import argparse
import datetime
import os
from shutil import rmtree

TEACHERS = {
    "blomnicki": "Benjamin Lomnicki",
    "melhachem": "Marcel El-Hachem",
    "palvarado": "Patricio Alvarado",
    "anuttall": "Angela Nuttall",
    "mstojoski": "Maria Stojoski",
    "tgould": "Thomas Gould",
    "gcrowfield": "Grahame Crowfield",
    "amakam": "Albert Makram"
}


def parseargs():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--conf-folder",
        default=os.path.join(os.path.expanduser("~"), ".edumated"),
        help="Config folder containg calender id, username, etc. (Default: ~/.edumated/",
    )
    parser.add_argument(
        "--start-date",
        default=datetime.datetime.now().date(),
        type=datetime.date.fromisoformat,
        help="Start date in ISO format, YYYY-MM-DD (Default: today)",
    )
    parser.add_argument(
        "--end-date",
        type=datetime.date.fromisoformat,
        help="End date in ISO format, YYYY-MM-DD",
    )
    parser.add_argument("--weeks", type=int, help="Weeks including start date to fetch")
    parser.add_argument(
        "--days",
        default=1,
        type=int,
        help="Days including start date to fetch (Default: 1)",
    )

    parser.add_argument(
        "--colour-help",
        action="store_true",
        help="Returns help for colour config and quits",
    )

    parser.add_argument("--reset", action="store_true", help="Deletes config files")

    args = parser.parse_args()

    colour_conf_file = os.path.join(args.conf_folder, "colour_config.txt")

    if not args.end_date:
        args.end_date = (
            args.start_date
            + datetime.timedelta(days=args.days)
            - datetime.timedelta(days=1)
        )

        if args.weeks:
            args.end_date = (
                args.start_date
                + datetime.timedelta(weeks=args.weeks)
                - datetime.timedelta(days=1)
            )

    if args.reset:
        rmtree(args.conf_folder)

    if not os.path.isdir(args.conf_folder):
        os.mkdir(args.conf_folder)

    if not os.path.isfile(colour_conf_file):
        with open(colour_conf_file, "w+") as file:
            for line in default_colours.items():
                file.write(f"{line[0]}: {line[1]}\n")

    if args.colour_help:
        print(
            f"""To add custom colours edit colour_config.txt in config folder ({args.conf_folder})
run edumated to generate folder and file

    Format is:
        SUBJECT: COLOUR

    SUBJECT: the first word of a subject (eg. English Advanced : English, Food Technology : Food, ect.)
            also includes "default" and "event" options
    
    COLOUR: google calendar colour names

            Google Colour options include:
                tomato, flamingo, tangerine, banana, sage, basil, peacock, blueberry, lavender, grape, graphite"""
        )
        if os.name == "nt":
            os.startfile(colour_conf_file)
        quit()

    return args


def daterange(start_date, end_date):
    dates = []
    for n in range(int((end_date - start_date).days) + 1):
        dates.append(start_date + datetime.timedelta(n))
    return dates


def extract_teacher(links):
    if "href" in links[0].keys():
        *_, email = links[0]["href"].split("=")
        abreviated, *_ = email.split("@")
        return TEACHERS.get(abreviated, abreviated)
    else:
        return None



from concurrent import futures
import datetime
from os import path
import pickle

from google.auth.transport.requests import Request
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from rfc3339 import rfc3339
from tqdm import tqdm


class CalendarTool:

    # If modifying these scopes, delete the file token.pickle.
    SCOPES = ["https://www.googleapis.com/auth/calendar"]

    def __init__(self, calendar_id, conf_folder):
        """Shows basic usage of the Google Calendar API.
        Prints the start and name of the next 10 events on the user's calendar.
        """
        self.CAL_ID = calendar_id
        self.CRED_FILE = {
            "installed": {
                "client_id": "260443697659-7fqv4qks7t7536sacm81ujk7n4bjfssf.apps.googleusercontent.com",
                "project_id": "edumated-1547966773995",
                "auth_uri": "https://accounts.google.com/o/oauth2/auth",
                "token_uri": "https://oauth2.googleapis.com/token",
                "auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
                "client_secret": "GgYorkYBYUqfMBcp81HbFYpB",
                "redirect_uris": ["urn:ietf:wg:oauth:2.0:oob", "http://localhost"],
            }
        }

        self.pickle = path.join(conf_folder, "token.pickle")
        creds = None
        # The file token.pickle stores the user's access and refresh tokens, and is
        # created automatically when the authorization flow completes for the first
        # time.
        if path.exists(self.pickle):
            with open(self.pickle, "rb") as token:
                creds = pickle.load(token)
        # If there are no (valid) credentials available, let the user log in.
        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_config(self.CRED_FILE, self.SCOPES)
                creds = flow.run_local_server()
            # Save the credentials for the next run
            with open(self.pickle, "wb") as token:
                pickle.dump(creds, token)

        self.service = build("calendar", "v3", credentials=creds)

        self.cal_events = []
        page_token = None
        while True:
            events = (
                self.service.events()
                .list(calendarId=self.CAL_ID, pageToken=page_token)
                .execute()
            )
            self.cal_events += events["items"]
            page_token = events.get("nextPageToken")
            if not page_token:
                break

    def make_events(self, events_data):
        try:
            date = events_data[0]["end_time"]
        except:
            return
        self.del_day(date)
        for event in events_data:
            self.make_event(event)

    def del_day(self, date):
        for event in self.cal_events:
            time_dict = event["start"]
            if "dateTime" in time_dict.keys():
                event_date = datetime.datetime.strptime(
                    event["start"]["dateTime"].split("+")[0], "%Y-%m-%dT%H:%M:%S"
                ).date()
            elif "data" in time_dict.keys():
                event_date = datetime.datetime.strptime(
                    event["start"]["date"].split("+")[0], "%Y-%m-%d"
                ).date()

            if event_date == date.date():
                self.service.events().delete(
                    calendarId=self.CAL_ID, eventId=event["id"]
                ).execute()

    def make_event(self, event_data):
        timezone = event_data["timezone"]

        event = {
            "summary": event_data["name"],
            "location": event_data["room"],
            "description": event_data["description"],
            "colorId": event_data["colour"],
            "start": {
                "dateTime": rfc3339(event_data["start_time"]),
                "timeZone": timezone,
            },
            "end": {"dateTime": rfc3339(event_data["end_time"]), "timeZone": timezone},
        }
        event = (
            self.service.events().insert(calendarId=self.CAL_ID, body=event).execute()
        )

    def make_all_events(self, dates):
        # with futures.ThreadPoolExecutor(max_workers=10) as executor:
        #     to_do = []
        #     for date in dates:
        #         future = executor.submit(
        #             self.make_events, date
        #         )
        #         to_do.append(future)

        #     for future in tqdm(
        #         futures.as_completed(to_do), unit="day", total=len(dates)
        #     ):
        #         pass
        for date in tqdm(dates):
            self.make_events(date)



import setuptools
from edumated.__version__ import __author__, __title__, __version__

setuptools.setup(
    name=__title__,
    version=__version__,
    author=__author__,
    author_email="",
    description="When you've check mated edumate",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Ninjakow/edumated",
    install_requires=open("requirements.txt").readlines(),
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={"console_scripts": ["edumated=edumated.edumate:main"]},
)



#!/usr/bin/env python3
from getpass import getpass
from os import path, mkdir

from .fetch import Fetcher
from .googcal import CalendarTool
from .util import daterange, parseargs


def main():
    args = parseargs()

    conf_file = path.join(args.conf_folder, "conf")

    if not path.isdir(args.conf_folder):
        mkdir(args.conf_folder)

    if not path.isfile(conf_file):
        username = input("Username: ")
        password = getpass(prompt="Password: ").replace("%", "%25")
        calendar_id = input("Google Calendar ID: ")
        with open(conf_file, "w") as f:
            f.write("\n".join([username, password, calendar_id]))
    else:
        with open(conf_file) as f:
            username, password, calendar_id = map(lambda l: l.strip(), f.readlines())

    calendar = CalendarTool(calendar_id, args.conf_folder)
    fetcher = Fetcher(username, password, args.conf_folder)
    dates = daterange(args.start_date, args.end_date)

    print(f"Fetching Events: {dates[0]} to {dates[-1]}")
    classes = fetcher.get_simple_dates(dates)

    print("Adding to Google Calendar")
    calendar.make_all_events(classes)
